%% INITIALIZATION
clear all; % clear the workspace from all previously stored variables and parameters
% data from main
steptime= 200; % Time when to perform a step-change [s]
step_waterlevel = 0.2; % Step in the water level [m]
step_inflow_liquid = 0.1;
step_inflow_gas = 0.1;               % Time when to perform a step-change [s]; for now 3 step-changes are conducted in the simulation
step_waterlevel = 0.2;          % Step in the water level [m]

% Dimensions and fluid properties
r = 1.65;                       % Radius of the separator [m]
L = 10;                         % Length of the active separation zone [m]
R = 8.31;                       % Universal gas constant [kg m^2/s^2/mol/K]
T = 328.5;                      % Temperature (assumed constant) [K]
density_gas = 49.7;             % Density of the gas phase [kg/m^3]
density_oil = 831.5;            % Density of the oil phase [kg/m^3]
density_water = 1030;           % Density of the water phase [kg/m^3]
MG = 0.01604;                   % Molar mass constant of the gas [kg/mol]
dyn_visc_water = 0.5e-3;        % Dynamic viscosity of the water phase [kg/m/s]
dyn_visc_oil = 1e-3;            % Dynamic viscosity of the water phase [kg/m/s]

% Controller parameters
tau_water = 5;                  % Water controller parameter
t_water = 0;                    % Low pass filter time constant for the  water controller
tau_liquid = 5;                 % Oil controller parameter
t_liquid = 0;                   % Low pass filter time constant for the oil controller
tau_gas = 5;                    % Gas controller parameter
t_gas = 0;                      % Low pass filter time constant for the gas controller
roc_water = 0.05;               % Maximal rate of change of the water controller 
roc_liquid = 0.05;              % Maximal rate of change of the liquid controller
roc_gas = 0.05;                 % Maximal rate of change of the gas controller

% Initial values for the integrators of the dynamic states
hWinit = 1;                     % Initial value for the water level [m]
hLinit = 2.5;                   % Initial value for the liquid level [m]
pinit = 68.7;                   % Initial pressure [bar]

% Define intital separation efficiencies in the inlet zone of the separator
phi_wo = 0.3; % Fraction of water going into the continuous oil phase - perfect separation for phi_wo = 0;
phi_ow = 0.3; % Fraction of oil going into the continuous water phase - perfect separation for phi_ow = 0;


%% Different Droplet Sizes and Distributions (Comment out, except for the case you want to simulate)

load('init_distribution.mat') % Load the pre-computed initial distributions for water in oil and oil in water
droplet_diameters_w = rw*2;
distribution_vector_w = fnw; % The distribution vector can be adjusted by adding a factor
droplet_diameters_o = ro*2;
distribution_vector_o = fno; % The distribution vector can be adjusted by adding a factor


% Calculation of the volume for one droplet of each droplet class, as well as their respective rise and settling velocities
volume_vector_o = zeros(1,length(droplet_diameters_o)); % Initialization of vector
volume_vector_w = zeros(1,length(droplet_diameters_w));
s_o = zeros(1,length(droplet_diameters_o)); % Initialization of vector
s_w = zeros(1,length(droplet_diameters_w)); % Initialization of vector
for i = 1:length(droplet_diameters_w)
    volume_vector_o(i) = 4/3*pi*(droplet_diameters_o(i)/2)^3; % Volumes of droplets [m^3]
    volume_vector_w(i) = 4/3*pi*(droplet_diameters_w(i)/2)^3; % Volumes of droplets [m^3]
    s_o(i) = abs((droplet_diameters_o(i))^2*9.81*(density_water-density_oil)/18/dyn_visc_water); % Rise velocities of oil droplets [m/s]
    s_w(i) = abs((droplet_diameters_w(i))^2*9.81*(density_oil-density_water)/18/dyn_visc_oil); % Settling velocities of water droplets [m/s]
end

% The volumes of droplets for oil and water are the same for now
vol_droplets_o = volume_vector_o; % Define vector of oil droplet volumes; UNIT m^3
vol_droplets_w = volume_vector_w; % Define vector of water droplet volumes; UNIT m^3

% The distributions of droplets for oil and water are the same for now
dist_droplets_o = distribution_vector_o; % Define vector of oil droplet distribution; UNIT #drops/(m*s)
dist_droplets_w = distribution_vector_w; % Define vector of water droplet distribution; UNIT #drops/(m*s)

% The total volumes for oil and water are the same for now; In the simulation, the total volumes are adapted by the factors F_o and F_w, respectively
volume_oil = (vol_droplets_o*dist_droplets_o'); % Define total volume of oil; UNIT m * #drops/(m*s) * m^3 = m^3/s
volume_water = (vol_droplets_w*dist_droplets_w'); % Define total volume of water; UNIT m * #drops/(m*s) * m^3 = m^3/s

% figure; % Plot the droplet distribtion
% plot(droplet_diameters_o*1e6,distribution_vector_o)
% title('Oil droplet distribution')
% axis([droplet_diameters_o(1)*1e6 droplet_diameters_o(end)*1e6 min(distribution_vector_o) 1.05*max(distribution_vector_o)])
% xlabel('Droplet diameters [{\mu}m]')
% ylabel('Number of droplets')
% 
% figure; % Plot the droplet distribtion
% plot(droplet_diameters_w*1e6,distribution_vector_w)
% title('Water droplet distribution')
% axis([droplet_diameters_w(1)*1e6 droplet_diameters_w(end)*1e6 min(distribution_vector_w) 1.05*max(distribution_vector_w)])
% xlabel('Droplet diameters [{\mu}m]')
% ylabel('Number of droplets')

%% DEFINE SIMULATION CASES (Comment out, except for the case you want to simulate)
% Beginning of the reservoir lifetime
alpha = 0.135;	% Water cut
beta = 0.865;	% Oil cut
qLin = 0.59;	% Liquid volumetric inflow [m^3/s]
qGin = 0.456;	% Constant inflow of gas in [m^3/s]

% Towards the end of the reservoir lifetime
% alpha = 0.475;    % Water cut
% beta = 0.525;     % Oil cut
% qLin = 0.73;      % Liquid volumetric inflow [m^3/s]
% qGin = 0.456;     % Constant inflow of gas in [m^3/s]


% %% SIMULATE
% steptime= 200;                  % Time when to perform a step-change [s]; for now 3 step-changes are conducted in the simulation
% step_waterlevel = 0.2;          % Step in the water level [m]
% step_inflow_liquid = 0.1;
% step_inflow_gas = 0.1;               % Time when to perform a step-change [s]; for now 3 step-changes are conducted in the simulation
% step_waterlevel = 0.2;          % Step in the water level [m]
% t_sim = time_length;             % Simulation time [s]
% tic
% sim('gravityseparator',t_sim)   % Simulate the Simulink model 'gravityseparator.slx' (has to be in the same folder as this file)
% toc
%% PLOT
% plotting; % Plot results using the m-file 'plotting.m' (has to be in the same folder as this file)
 
%% variable input
time_length = 15000; %simulation time
sample_time_u = 100; %hold time for input
n_inputs = time_length/sample_time_u; % how many different inputs
sample_time_sys = 10; %sample stime for system
training_samples = time_length/sample_time_sys;

alpha_init = 0.065;
q_low = 0.00; %lowest inflow
q_max = 0.135*2; %highest inflow
alpha_change.time=sample_time_u*(0:n_inputs-1)';
alpha_change.signals.values =q_low + rand(n_inputs,1)*(q_max-q_low);
sample_time_u2=50;
n_inputs2=time_length/sample_time_u2;
alpha_change2.time=sample_time_u2*(0:n_inputs2-1)';
alpha_change2.signals.values =rand(n_inputs2,1)*((q_max-q_low)/1.2);
%alpha_change2.signals.values([1:500 987:1234 1750:2000])=0;
alpha_change2.signals.values([1:time_length/(2*sample_time_u2)])=0;

sample_time_ref = 200;
n_inputs_ref = time_length/sample_time_ref;
sample_time_ref2=50;
n_inputs_ref2=time_length/sample_time_ref2;
hw_lvl_rand.time=sample_time_ref*(0:n_inputs_ref-1)';
hw_lvl_c=1;
hw_lvl_low = 0;
hw_lvl_max = 0;%0.8;
hw_lvl_rand.signals.values =hw_lvl_low + rand(n_inputs_ref,1)*(hw_lvl_max-hw_lvl_low);

q_lin_rand1.time=sample_time_ref*(0:n_inputs_ref-1)';
q_lin_c=0.44;
q_lin_low = 0;
q_lin_max = 0.20;%0.050;
q_lin_rand1.signals.values =q_lin_low + rand(n_inputs_ref,1)*(q_lin_max-q_lin_low);
q_lin_rand2.time=sample_time_ref2*(0:n_inputs_ref2-1)';
q_lin_rand2.signals.values =rand(n_inputs_ref2,1)*((q_lin_max-q_lin_low)/1.3);
q_lin_rand2.signals.values([1:time_length/4/sample_time_ref2, 3/5*time_length/sample_time_ref2:4/5*time_length/sample_time_ref2])=0;

q_gin_rand1.time=sample_time_ref*(0:n_inputs_ref-1)';
q_gin_c=0.306;
q_gin_low = 0;
q_gin_max = 0.20;
q_gin_rand1.signals.values =q_gin_low + rand(n_inputs_ref,1)*(q_gin_max-q_gin_low);
q_gin_rand2.time=sample_time_ref2*(0:n_inputs_ref2-1)';
q_gin_rand2.signals.values =rand(n_inputs_ref2,1)*((q_gin_max-q_gin_low)/1.3);
q_gin_rand2.signals.values([1:time_length/4/sample_time_ref2 time_length*4/5/sample_time_ref2:time_length/sample_time_ref2])=0;
