%% Plot 1
figure;
subplot(3,2,1) % Plot the actual and desired water levels
plot(hW.time,hW.signals.values,'b')
hold on
plot(hWdes.time,hWdes.signals.values,'r')
ylabel('[m]')
title('Water level h_W')
axis([0 hW.time(end) .8 1.8])

subplot(3,2,2) % Plot the water outflow
plot(qWout.time,qWout.signals.values)
ylabel('[m^3 s^{-1}]')
title('Water outflow q_{W,out}')

subplot(3,2,3) % Plot the actual and desired liquid levels
plot(hL.time,hL.signals.values,'b')
hold on
plot(hLdes.time,hLdes.signals.values,'r')
ylabel('[m]')
title('Liquid level h_L')
axis([0 hL.time(end) 2.2 2.8])

subplot(3,2,4) % Plot the oil outflow
plot(qOout.time,qOout.signals.values)
ylabel('[m^3 s^{-1}]')
title('Oil outflow q_{O,out}')

subplot(3,2,5) % Plot the pressure
plot(p.time,p.signals.values,'b')
hold on
plot(pdes.time,pdes.signals.values,'r')
xlabel('Time [s]')
ylabel('[bar]')
title('Gas pressure')
axis([0 p.time(end) 65 72])

subplot(3,2,6) % Plot the gas outflow
plot(qGout.time,qGout.signals.values)
xlabel('Time [s]')
ylabel('[m^3 s^{-1}]')
title('Gas outflow q_{G,out}')

%% Plot 2
figure;
subplot(2,1,1) % Plot the separation efficiencies for oil and water
plot(efficiency_water.time,efficiency_water.signals.values*100,'b')
hold on
plot(efficiency_oil.time,efficiency_oil.signals.values*100,'r')
ylabel('%')
title('Efficiencies')

subplot(2,1,2) % Plot the correction factors for oil and water droplets
plot(factor_water.time,factor_water.signals.values,'b')
hold on
plot(factor_oil.time,factor_oil.signals.values,'r')
xlabel('Time [s]')
ylabel('-')
title('Factors F_W (blue) and F_O (red)')


%% Plot 3
figure;  % Plot the oil droplet distributions at the beginning and at the end of the active separation zone
surf(droplet_diameters_o*1e6,n_5.time(1:end),n_5.signals.values(1:end,:),'EdgeColor','none','LineStyle','none','FaceLighting','phong')
view([10 20]);
title('Distribution of oil droplets at the water outlet')
xlabel('Droplet diameter [{\mu}m]')
ylabel('Time [s]')
zlabel('Number of droplets')


%% Plot 4

% Create vectors
nwater = zeros(length(nW_5.time),length(droplet_diameters_w));
oil_init = zeros(length(nW_5.time),length(droplet_diameters_w));
water_init = zeros(length(nW_5.time),length(droplet_diameters_w));
for j=1:length(droplet_diameters_w)
    for i=1:length(nW_5.time)
        nwater(i,j)=nW_5.signals.values(1,j,i);
        oil_init(i,j)=dist_droplets_o(j);
        water_init(i,j)=dist_droplets_w(j);
    end
end

figure; % Plot the water droplet distributions at the beginning and at the end of the active separation zone
surf(droplet_diameters_w*1e6,nW_5.time(1:end),nwater,'EdgeColor','none','LineStyle','none','FaceLighting','phong')
view([10 20]);
title('Distribution of water droplets at the oil outlet')
xlabel('Droplet diameter [{\mu}m]')
ylabel('Time [s]')
zlabel('Number of droplets')

